# combo - content management system
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import requests
import urllib

from django.conf import settings
from django.db import models
from django import template
from django.utils.translation import ugettext_lazy as _

from combo.data.models import CellBase
from combo.data.library import register_cell_class
from combo.utils import NothingInCacheException, sign_url

from .utils import get_wcs_services, get_wcs_data


@register_cell_class
class FamilyInfosCell(CellBase):
    template_name = 'family/infos.html'
    user_dependant = True

    class Meta:
        verbose_name = _('Family Information Cell')

    class Media:
        js = ('xstatic/jquery-ui.min.js', 'js/gadjo.js',)
        css = {'all': ('xstatic/themes/smoothness/jquery-ui.min.css', )}

    @classmethod
    def is_enabled(cls):
        return getattr(settings, 'FAMILY_SERVICE', None)

    def get_cell_extra_context(self):
        context = {}
        if (self.context.get('user') and self.context.get('request') and
                getattr(self.context['request'], 'session', {}).get('mellon_session')):
            name_id = self.context.get('request').session['mellon_session'].get('name_id_content')
            if name_id:
                params = {'NameID': name_id, 'orig': self.context['request'].get_host()}
                # get family data
                ws_family_url = sign_url(settings.FAMILY_SERVICE.get('url') +
                                         '/family/?' + urllib.urlencode(params),
                                         settings.FAMILY_SERVICE.get('signature_key'))
                ws_family_data = requests.get(ws_family_url).json()
                if ws_family_data.get('err') or not ws_family_data.get('data'):
                    return {'family': ws_family_data}
                # get formdefs and formdata from wcs
                def add_forms(obj, formdefs, forms, drafts, id_varname):
                    if obj.get('keywords'):
                        keywords_set = set(obj['keywords'])
                        obj['formdefs'] = [f for f in formdefs if f.get('keywords')
                                           and keywords_set.issuperset(f['keywords'])]
                    if obj.get('id'):
                        obj['forms'] = [f for f in forms
                                        if f['fields'].get(id_varname) == obj['id']]
                        obj['drafts'] = [f for f in drafts
                                         if f['fields'].get(id_varname) == obj['id']]
                if get_wcs_services():
                    formdefs = get_wcs_data('/api/formdefs/', params)
                    params['full'] = 'on'
                    forms = get_wcs_data('/api/user/forms', params)
                    drafts = get_wcs_data('/api/user/drafts', params)
                    family = ws_family_data['data']
                    add_forms(family, formdefs, forms, drafts, id_varname='family_raw')
                    for adult in family.get('adults', []):
                        add_forms(adult, formdefs, forms, drafts, id_varname='adult_raw')
                    for child in family.get('children', []):
                        add_forms(child, formdefs, forms, drafts, id_varname='child_raw')
                    for contact in family.get('contacts', []):
                        add_forms(contact, formdefs, forms, drafts, id_varname='contact_raw')
                context['family'] = ws_family_data
        return context

    def render(self, context):
        self.context = context
        if not context.get('synchronous'):
            raise NothingInCacheException()
        return super(FamilyInfosCell, self).render(context)
